@@ -1,6 +1,16 @@
 import subprocess, re, sys
 from subprocess import DEVNULL
 from typing import Optional, Any
+import xml.etree.ElementTree as ET
+
+def query_yn(question: str) -> bool:
+    while True:
+        print(question + ' [y/n]', end=' ')
+        choice = input().lower()
+        if choice == 'y' or choice == 'yes':
+            return True
+        elif choice == 'n' or choice == 'no':
+            return False
 
 def apply_vars(text: str, vars: dict[str,Any]) -> str:
     for k, v in vars.items():
@@ -31,8 +41,8 @@ def inkscape_run(args: list[str]) -> int:
 def pattern_get(pattern: str, string: str, group: int) -> str:
     match = re.search(pattern, string)
     if not match: raise ValueError(f'No match found by "{pattern}"')
-    g = match.group(1)
-    if not g: raise ValueError(f'Match group {group} is empty')
+    g = match.group(group)
+    if g is None: raise ValueError(f'Match group {group} is None')
     return g
 
 def number_of_pages(filename: str) -> int:
@@ -59,3 +69,29 @@ def parse_range(text: str, num_pages: int) -> set[int]:
             continue
         raise ValueError(f'Invalid page range: {text}')
     return pages
+
+def find_elements_by_class(tree: ET.ElementTree, cls: str) -> ET.Element:
+    result = []
+    for el in tree.iter():
+        if cls in el.get('class', ''):
+            result.append(el)
+    return result
+
+def px(length: str) -> float:
+    match = re.search(r'([0-9.]+)\s*([a-zA-Z%]*)', length)
+    num = match.group(1)
+    unit = match.group(2)
+    if   unit == 'cm':
+        return float(num) * 37.795
+    elif unit == 'mm':
+        return float(num) * 3.7795
+    elif unit == 'in':
+        return float(num) * 96.0
+    elif unit == 'pc':
+        return float(num) * 16.0
+    elif unit == 'pt':
+        return float(num) * 1.333333333
+    elif unit == 'px' or unit == '':
+        return float(num)
+    else:
+        raise ValueError(f'Invalid length: {length}')
